/* $Id: addrlib.h,v 1.1.4.2 2007/06/09 15:20:00 hurtta Exp $ */

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 1.1.4.2 $   $State: Exp $
 *
 *  Author: Kari Hurtta <hurtta+elm@posti.FMI.FI>
 *      or  Kari Hurtta <elm@elmme-mailer.org>
 *****************************************************************************/

struct addr_list;
struct address;


/* address list operations ------------------------------------------------- */

extern struct addr_list  * new_empty_addr_list P_((int alloced));

extern void free_addr_list P_((struct addr_list **list));

extern struct addr_list * parse_header_address P_((const char *headername,
						   const char *buffer, 
						   int demime, 
						   charset_t defcharset,
						   struct header_errors **header_error));

extern struct addr_list * dup_addr_list P_((const struct addr_list *list));

extern void append_addr_list P_((struct addr_list *list,
				 const struct addr_list *src));


extern int addr_list_item_count   P_((const struct addr_list *list));
extern int addr_list_group_count  P_((const struct addr_list *list));

/* *group_num == -1 if not part of group */
extern const struct address * addr_list_get_item P_((const struct addr_list 
						     *list,
						     int idx,
						     int *group_num));
extern const struct string * addr_list_get_group P_((const struct addr_list 
						     *list,
						     int idx));


extern void addr_list_replace_item P_((struct addr_list *list,
				       int idx,
				       const struct address *addr,
				       int group_num));

/* groupidx == -1 if not part of group */
extern int add_address_to_list P_((struct addr_list *list,
				   const struct address *addr,
				   int groupidx));

/* Returns groupidx which can be used with add_address_to_list */
extern int add_group_to_list P_((struct addr_list *list,
				 const struct string *phrase));

/* Return -1 if group not found */
extern int addr_list_lookup_group P_((struct addr_list *list,
				      const struct string *group,
				      int create));

/* address operations ------------------------------------------------------ */

extern void free_address P_((struct address **addr));

extern struct address * new_address P_((const char * ascii_addr,
					const struct string  * phrase,
					const struct string  * comament));

extern const struct string * address_get_phrase P_((const struct address *
						    addr));
extern const struct string * address_get_comment P_((const struct address *
						     addr));
extern const char * address_get_ascii_addr P_((const struct address *addr));

/* mailing list  ------------------------------------------------------ */

struct mlist_info;
struct mlist_conf;

typedef int mlist_match_id_f P_((char * list_id,
				 struct list_info *list));

extern struct mlist_scan * mlist_get_matches P_((struct mlist_conf *list,
						 struct header_rec *hdr,
						 mlist_match_id_f *f));

extern void free_mlist_scan P_((struct mlist_scan **match));

extern void append_mlist_scan P_((struct mlist_scan *list,
				  const struct mlist_scan *other));

/* List is non-empty */
extern int have_mlist_scan P_((struct mlist_scan *match));

extern const struct mlist_info * get_mlist_info P_((struct mlist_scan *match));

/* Display address --------------------------------------------------------- */

struct header_rec;  
struct string;

typedef void decode_who P_((int class, char *ptr, int size));
extern int DisplayAddress  P_((struct header_rec *h,struct string **f));

/* okay_addr.c ------------------------------------------------------------- */

extern int okay_address_l P_((const struct address  *address, 
			      const struct addr_list *return_address));

/* mailer.c    ------------------------------------------------------------- */

struct mailing_headers;

extern char **argv_from_headers   P_((struct mailing_headers * headers));

extern void free_argv P_((char ***argv));

/* headers.c   ------------------------------------------------------------- */

#define   RHL_MARK_FOLDING      1
#define   RHL_CHECK_HEADER      2

extern int read_header_line             P_((FILE *,char *,int,int));
extern int state_read_hdr_line          P_((in_state_t *,char *, int, int));

typedef struct header_info * header_ptr;

/* Returns pointer to static storage */
extern const char * give_header_name P_((header_ptr X));
extern int is_mailing_header P_((header_ptr X));

extern int add_to_mailing_header P_((struct mailing_headers * headers,
				     header_ptr X,const char *value,
				     int demime,
				     charset_t defcharset,
				     int replace
				     ));

#define HEADER_magic        0xFC00

typedef struct header_list {
    unsigned short magic;
    header_ptr header_name;
    struct header_list * next_this_header;
    struct header_list * next_header;
    struct header_list * next_other_header;
    char * body;
} * header_list_ptr;

extern void update_header_list P_((header_list_ptr *list,
				   header_list_ptr *last1,
				   const char * name,
				   const char * value));

extern header_ptr find_header  P_((const char *name, int create_flag));
extern header_list_ptr locate_header      P_((header_list_ptr,header_ptr));

#define locate_header_by_name(h,n)      locate_header(h,find_header(n,0))
extern void delete_headers              P_((header_list_ptr *hdr));


extern struct string * give_decoded_header P_((header_list_ptr tmphdr,
					       int demime, 
					       charset_t defcharset));


/* outheaders.c ------------------------------------------------------------ */

struct textual {
    struct string *Textual;
    int pos;
    int len;
};

struct expanded_address {
    unsigned short magic;               /* EXP_ADDR_magic */
    struct addr_list  *addrs;           
    
    struct textual *surface;
    int surface_len;
};

extern void dump_expanded_address P_((int debuglevel, const char *text,
				      struct expanded_address expanded));

extern void expanded_address_from_list P_((struct expanded_address *result,
					    const struct addr_list *addr_list));


struct mailing_headers {
    unsigned short magic;                  /* MAIL_HDR_magic */

    struct string *subject;
    struct expanded_address from;
    struct id_phrase in_reply_to;

    char *expires;
    char *action; 
    char *priority;
    
    char *sender;

    struct expanded_address reply_to; 
    struct expanded_address to;
    struct expanded_address cc;
 
    struct user_header {
	header_ptr      name;
	struct string  *value;
    }  * user_header;
    int  user_header_count;

    struct expanded_address bcc;
    char *precedence;
    char *expires_days;    

    struct mailer_env_from  *env_from;
};

extern void import_mailheaders P_((struct mailing_headers * headers));

extern  int add_expanded_addr_ P_((struct expanded_address *x,
				   const char *ADR, 
				   const struct string *FN,
				   const struct string *COM));
extern int add_expanded_addr0_ P_((struct expanded_address *x,
				   const struct address *ADR,
				   int g));
extern int add_expanded_addr_g P_((struct expanded_address *x,
				   const struct address *ADR,
				   const struct string *group));

extern  int add_expanded_addr_4 P_((struct expanded_address *x,
				    const char *ADR, 
				    const struct string *FN,
				    const struct string *COM,
				    const struct string *group));


extern int add_group_phrase_ P_((struct expanded_address *x,
				 const struct string *GRP));

extern int add_textual_addr_ P_((struct expanded_address *x,
			  const struct string *TEXTUAL, 
			  int POS, int LEN));

#define ADD_EXPANDED(x,ADR,FN,COM)  add_expanded_addr_(&x,ADR,FN,COM)
#define ADD_EXPANDED0(x,ADR)        add_expanded_addr0_(&x,ADR,-1)
#define ADD_EXPANDED_g(x,ADR,G)        add_expanded_addr0_(&x,ADR,G)
#define ADD_EXPANDED_G(x,ADR,G)        add_expanded_addr_g(&x,ADR,G)
#define ADD_EXPANDED4(x,ADR,FN,COM,GRP) add_expanded_addr_4(&x,ADR,FN,COM,GRP)
#define ADD_GROUP(x,GRP)               add_group_phrase_(&x,GRP)
#define ADD_TEXTUAL(x,TEXTUAL,POS,LEN) add_textual_addr_(&x,TEXTUAL,POS,LEN)


extern void zero_expanded_address P_((struct expanded_address *x));
extern void free_expanded_address P_((struct expanded_address *x));
extern void copy_expanded_address P_((struct expanded_address *result,
				      struct expanded_address source));

extern struct string * make_surface_addr P_((const struct address * address));
extern int check_8bit_str P_((char *str));    

/* write_hdr.c ------------------------------------------------------------- */

extern void write_addr_header P_((out_state_t *mailer,
				  char *hdr_name,
				  struct addr_list * addr,
				  int top_encoding,
				  int enmime,
				  charset_t hdr_charset,
				  int add_group));


/* error.c ----------------------------------------------------------------- */

extern void free_header_errors P_((struct header_errors **error));

extern void process_header_error P_((struct header_errors **error,
				     const char * format, const char *msg, 
				     ...));

extern int get_header_errors_count P_((struct header_errors *error));
extern const struct string * get_header_errors_msg P_((struct header_errors *
						       error, int idx));

extern void print_header_errors P_((struct header_errors *error));



/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */
