static char rcsid[] = "@(#)$Id: alias_pattern.c,v 1.4.8.3 2007/04/05 19:44:58 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 1.4.8.3 $   $State: Exp $
 *
 *  Author: Kari Hurtta <hurtta+elm@posti.FMI.FI> 
 *      or  Kari Hurtta <elm@elmme-mailer.org>
 ******************************************************************************
 * Inludes code from Elm 2.4 src/pattern.c which have following copyright:
 *
 *  The Elm Mail System 
 *
 *			Copyright (c) 1988-1992 USENET Community Trust
 *			Copyright (c) 1986,1987 Dave Taylor
 ****************************************************************************/

#include "def_mcommon.h"

static unsigned char *s2us P_((char *str));
static unsigned char *s2us(str) 
     char *str;
{
    return (unsigned char *)str;
}

static const unsigned char *cs2us P_((const char *str));
static const unsigned char *cs2us(str) 
     const char *str;
{
    return (const unsigned char *)str;
}



static int name_matches P_((struct alias_rec * a, struct string *pat));
static int alias_matches P_((struct alias_rec * a, struct string *pat));
static int comment_matches P_((struct alias_rec * a, struct string *pat));
static int address_matches P_((struct alias_rec * a, struct string *pat,
			       struct AliasView *aview));

int mc_matches_alias(u,idx, meta_pattern)
     union mcommon_union *u;
     int idx;
     struct string * meta_pattern;
{
    struct AliasView *aview = u->alias.aw;
    struct alias_rec *a = give_alias(aview,idx);
    int r = 0;
    
    if (a) {
	r = name_matches(a, meta_pattern) || alias_matches(a, meta_pattern); 
    }

    return r;
}

static int name_matches(a, pattern)
     struct alias_rec * a;
     struct string * pattern;
{
    /** Returns true iff the pattern occurs in it's entirety
	in the proper name of the indicated alias **/
    
    int r = 0;

    if (a) {
	struct string *s = new_string2(display_charset,
				       s2us(a->name));

	if (find_pattern_from_string(s,pattern,1))
	    r++;

	free_string(&s);
    }

    return r;
}

static int alias_matches(a, pattern)
     struct alias_rec * a;
     struct string * pattern;
{
    /** Returns true iff the pattern occurs in it's entirety
	in the alias name of the indicated alias **/
    
    int r = 0;
    
    if (a) {
	struct string *s = new_string2(display_charset,
				       s2us(a->alias));
	
	if (find_pattern_from_string(s,pattern,1))
	    r++;
	
	free_string(&s);
    }

    return r;
}


static int comment_matches(a, pattern)
     struct alias_rec * a;
     struct string * pattern;
{
    /** Returns true iff the pattern occurs in it's entirety
	in the comment field of the indicated alias **/

    int r = 0;
    
    if (a) {
	struct string *s = new_string2(display_charset,
				       s2us(a->comment));
	
	if (find_pattern_from_string(s,pattern,1))
	    r++;
	
	free_string(&s);
    }
    return r;

}

static int address_matches(a,pattern, aview)
     struct alias_rec * a;
     struct string * pattern;
     struct AliasView *aview;
{
    struct addr_list *result;
    int dummy;
    int res = 0;
        
    /** Returns true iff the pattern occurs in it's entirety
	in the fully expanded address of the indicated alias **/
    
    if (a) {
	result = get_alias_address_l(a->alias,TRUE,
				     &dummy, NULL, aview);

	if (result) {
	    int len = addr_list_item_count(result);
	    int glen = addr_list_group_count(result);
	    int idx;

	    for (idx = 0; idx < glen  && !res; idx++) {
		const struct string * group =
		    addr_list_get_group(result,idx);
	
	    	if (find_pattern_from_string(group,pattern,1))
		    res = 1;
	    }
	    
	    for (idx = 0; idx < len  && !res; idx++) {

		int group = -1;
		const struct address * address = 
		    addr_list_get_item(result,idx,&group);
		
		const char          * addr     = address_get_ascii_addr(address);
		const struct string * fullname = address_get_phrase(address);
		const struct string * comment  = address_get_comment(address);

		if (addr) {
		    struct string *s = new_string2(ASCII_SET,cs2us(addr));

		    if (find_pattern_from_string(s,pattern,1))
			res = 1;
		    
		    free_string(&s);
		}

		if (fullname) {
		    if (find_pattern_from_string(fullname,pattern,1))
			res = 1;
		}
		if (comment) {
		    if (find_pattern_from_string(comment,pattern,1))
		    res = 1;
		}
		
	    }

	    free_addr_list(&result);	
	}
    }

    return(res);
}

int mc_match_in_text_alias(u,meta_pattern, page, LOC)
     union mcommon_union *u;
     struct string * meta_pattern;
     struct menu_context  *page;
     struct screen_parts *LOC; 
{   
    struct AliasView *aview = u->alias.aw;

    /* From pattern_match() */

    int current = get_alias_current(aview);
    int ac = get_alias_count(aview);
    int selected = get_alias_selected(aview);
    int i;
    int ret = 0;

    for (i = current; i < ac; i++) {
	struct alias_rec * a = give_alias(aview,i);
	
	if (a && (
		  name_matches(a, meta_pattern) || 
		  alias_matches(a, meta_pattern) ||
		  comment_matches(a, meta_pattern) ||
		  address_matches(a, meta_pattern, aview)
		  )) {
	    
	    if (!selected || 
		(selected && a->status & VISIBLE)) {
		
		current = ++i;
		set_alias_current(aview,current);
		
		ret = 1;
		goto out;
	    }
	}
    }
 out:

    return ret;
}

/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */

